// SPDX-FileCopyrightText: 2018 - 2022 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

package common

func FindFile(name string) string {
	// TODO: no path finding heuristic yet
	return name
}
